/*
 ============================================================================
  Parser.h: A C++ LALR(1) parser driver with syntax error diagnostics.

  By Loren Cobb, Corrales Software, Corrales, NM 87048-2730.
  
  Copyright (c) 1989-93 by Abraxas Software Inc. (R).  All rights reserved.
 ============================================================================
*/  


#include "tparser.h"

int TParser::yyparse( void )
	{
	YYSTYPE         *yypvt,
				*yypv;
				
	short           yys[YYMAXDEPTH],
				yyj, yym, yystate,
				*yyps, yyn, *yyxi;

	yystate = 0;
	yytoken = -1;
	errct = 0;
	errfl = 0;
	yyps = yys - 1;
	yypv = yyv - 1;

yystack:                /* put a state and value onto the stack */

	 if ( ++yyps > yys + YYMAXDEPTH )
		{
		yymessage( FATAL, 0 );          /*      This condition is fatal */
		}
	 *yyps = yystate;
     ++yypv;
     *yypv = yyval;

yynewstate:

	yyn = yypact[yystate];

	if ( yyn <= YYFLAG )
		goto yydefault;         /* simple state with only default action */

	if ( yytoken < 0 )              /* no lookahead token? */
		{
		yytoken = yylex();
		if ( yytoken < 0 )
			yytoken = 0;
		}

	yyn += yytoken;
	if ( yyn < 0 || yyn >= YYLAST )         /*      no entry within table?                                  */
		goto yydefault;                                 /*      simple state with only default action   */

	if ( yychk[yyn = yyact[yyn]] == yytoken )               /*      valid shift?            */
		{
		yytoken = -1;                   /*      lookahead token is read */
		yyval = yylval;
		yystate = yyn;
		if ( errfl > 0 )                /*      still in error state?           */
			--errfl;
		goto yystack;
		}

yydefault:              /* simple state with only default action */

	if ( (yyn = yydef[yystate]) == -2 )
		{
		if ( yytoken < 0 )              /* no lookahead token? */
			{
			yytoken = yylex();
			if ( yytoken < 0 )
				yytoken = 0;
			}

		/*      Look through exception table:   */
		
		for ( yyxi = (short *)yyexca; *yyxi != -1 || yyxi[1] != yystate; yyxi += 2 )
			;               /* VOID */
		while ( *(yyxi += 2) >= 0 )
			{
			if ( *yyxi == yytoken )
				break;
			}
		if ( (yyn = yyxi[1]) < 0 )
			return 0;                               /*      accept  */
		}

	if ( yyn == 0 )                 /*      error?  */
		{
		switch( errfl )
			{
		case 0:                 /*      brand new error */
			yymessage( START, yytoken );
			yyn = yypact[yystate];
			if ( (yyn > YYFLAG) && (yyn < YYLAST) )
				{
				register int    x;

				for ( x = (yyn > 0) ? yyn : 0; x < YYLAST; ++x )
					if ( yychk[yyact[x]] == x - yyn )
						yymessage( EXPECT, x - yyn );
				}
			yymessage( SENDIT, 0 );
			++errct;

		case 1:
		case 2:         /*      Incompletely recovered error ... try again:             */
			errfl = 3;
					/*      Find a state where "error" is a legal shift action:     */
			while ( yyps >= yys )
				{
				yyn = yypact[*yyps] + YYERRCODE;
				if ( (yyn >= 0) && (yyn < YYLAST) && (yychk[yyact[yyn]] == YYERRCODE) )
					{
					yystate = yyact[yyn];   /*      Simulate a shift of "error"     */
					goto yystack;
					}
				yyn = yypact[*yyps];
				
				--yyps;         /*      The current yyps has no shift on "error",       */
				--yypv;         /*      so pop the stack.                                                       */
				}

			/*
				There is no state on the stack with an error shift,
				so abort:
			*/
yyabort:
			return 1;

		case 3:                                         /*      No shift yet; clobber input char:       */
			if ( yytoken == 0 )             /*      EOF?                                                            */
				goto yyabort;           /*      Don't discard EOF, quit instead.        */
			yytoken = -1;
			goto yynewstate;                /*      Try again in the same state     .               */
			}
		}

	/*      Reduction by production yyn:    */

	yyps -= yyr2[yyn];
	yypvt = yypv;
	yypv -= yyr2[yyn];
	yyval = yypv[1];
	yym = yyn;
    
	/*      Consult goto table to find next state:  */

	yyn = yyr1[yyn];
	yyj = yypgo[yyn] + *yyps + 1;
	if ( (yyj >= YYLAST) || (yychk[yystate = yyact[yyj]] != -yyn) )
		yystate = yyact[yypgo[yyn]];
	switch ( yym )
		{
		$A
		}
	goto yystack;   /*      Stack new state and value       */
	}

