#include <stdio.h>
#include "pcy_sk.hpp" 

extern char* yydisplay(int);

ABXYacc::ABXYacc(ABXLex* LexObject)
{
    lexObject = LexObject;
    yyValueStack = new YYSTYPE[YYMAXDEPTH];
    redSequence = new int[YYREDMAX];
    yyToken = -1;
    yyErrorFlag = 0;
    yyErrorCount = 0;
    redCount = 0;
}

ABXYacc::~ABXYacc()
{
    delete[] yyValueStack;
    delete[] redSequence;
}                                                               

int ABXYacc::yyParse(void)
{
#ifdef YYSHORT
    const short *yyxi;
#else
    const int *yyxi;
#endif
#ifdef YYASTFLAG
    int ti;
    int tj;
#endif

#ifdef YYDEBUG
int tmptoken;
#endif
    int statestack[YYMAXDEPTH]; /* state stack */
    int j,
	m;              /* working index */
    YYSTYPE *yypvt;
    int tmpstate,
	*yyps,
	n;
    YYSTYPE *yypv;


    tmpstate = 0;
    yyToken = -1;


#ifdef YYDEBUG
    tmptoken = -1;
#endif


    yyErrorCount = 0;
    yyErrorFlag = 0;
    yyps = statestack -1 ;
    yypv = yyValueStack - 1;

enstack:    /* push stack */
#ifdef YYDEBUG
    printf("at state %d, next token %d\n", tmpstate, tmptoken);
#endif
    if ( ++yyps - &statestack[YYMAXDEPTH-1] > 0)
    {
	yyerror("pcyacc internal stack overflow", 0, 0);
	return(1);
    }
    *yyps = tmpstate;
    ++yypv;
    *yypv = yyval;

newstate:
    n = yypact[tmpstate];
    if (n <= PCYYFLAG)
	goto defaultact; /*  a simple state */

    if (yyToken < 0)
	if ((yyToken=lexObject->yyLex()) < 0)
	    yyToken = 0;
	else if (yyToken>=0)    
      printf("tokenValue=[%d], actual token=[%s]\n",
	         yyToken, yydisplay(yyToken));  
    
    if ((n += yyToken) < 0 || n >= YYLAST)
	goto defaultact;


    if (yychk[n=yyact[n]] == yyToken)
    {    /* a shift */
#ifdef YYDEBUG
	tmptoken  = yyToken;
#endif
	yyToken = -1;
	yyval = yylval;
	tmpstate = n;
	if (yyErrorFlag > 0)
	    --yyErrorFlag;
	goto enstack;
    }

defaultact:


    if ((n=yydef[tmpstate]) == -2)
    {
	if (yyToken < 0)
	    if ((yyToken=lexObject->yyLex())<0)
		  yyToken = 0; 
	    else if (yyToken>=0)   
          printf("tokenValue=[%d], actual token=[%s]\n",
		         yyToken, yydisplay(yyToken)); 

	for (yyxi=yyexca; (*yyxi!= (-1)) || (yyxi[1]!=tmpstate); yyxi += 2)
	     ;
	while (*(yyxi+=2) >= 0)
	    if (*yyxi == yyToken)
		break;
	if ((n=yyxi[1]) < 0)
	{    /* an accept action */
#ifdef YYASTFLAG
	    yytfilep = fopen(yytfilen, "w");
	    if (yytfilep == NULL)
	    {
		fprintf(stderr, "Can't open t file: %s\n", yytfilen);
		return(0);          
	    }
	    for (ti=redcnt-1; ti>=0; ti--)
	    {
		tj = svdprd[redSequence[ti]];
		while (strcmp(svdnams[tj], "$EOP"))
		    fprintf(yytfilep, "%s ", svdnams[tj++]);
		fprintf(yytfilep, "\n");
	    }
	    fclose(yytfilep);
#endif

	    return (0);
	}
    }

    if (n == 0)
    {    /* error situation */
	switch (yyErrorFlag)
	{
	    case WAS0ERR:    /* an error just occurred */
		yyerror("Syntax error", yydisplay(yyToken), lexObject->get_yyText());
		if ((n=yypact[tmpstate])>PCYYFLAG && n<YYLAST)
		{
			register int x;
			for (x=(n>0)?n:0; x<YYLAST; ++x)
			{
				if (yychk[yyact[x]]==x-n && x-n!=YYERRCODE)
					yyerror(0, yydisplay(x-n), lexObject->get_yyText());
			}
			yyerror(0, 0, lexObject->get_yyText());
		}
		yyerror(0, 0, lexObject->get_yyText());
yyErrLab:
		++yyErrorCount;

	    case WAS1ERR:
	    case WAS2ERR:    /* try again */
		yyErrorFlag = 3;
		/* find a state for a legal shift action */
		while (yyps >= statestack)
		{
		    n = yypact[*yyps] + YYERRCODE;
		    if (n >= 0 && n < YYLAST && yychk[yyact[n]] == YYERRCODE)
		    {
			tmpstate = yyact[n];    /* simulate a shift of "error" */
			goto enstack;
		    }
		    n = yypact[*yyps];

		    /* the current yyps has no shift on "error", pop stack */
#ifdef YYDEBUG
		    printf("error: pop state %d, recover state %d\n", *yyps, yyps[-1]);
#endif
		    --yyps;
		    --yypv;
		}

yyabort:

#ifdef YYASTFLAG
		yytfilep = fopen(yytfilen, "w");
		if (yytfilep == NULL)
		{
		    fprintf(stderr, "Can't open t file: %s\n", yytfilen);
		    return(1);
		}
		for (ti=1; ti<redCount; ti++)
		{
		    tj = svdprd[redSequence[ti]];
		    while (strcmp(svdnams[tj], "$EOP"))
			fprintf(yytfilep, "%s ", svdnams[tj++]);
		    fprintf(yytfilep, "\n");
		}
		fclose(yytfilep);
#endif

		return(1);


	    case WAS3ERR:  /* clobber input char */
#ifdef YYDEBUG
		printf("error: discard token %d\n", yyToken);
#endif
		if (yyToken == 0) 
		    goto yyabort; /* quit */
		yyToken = -1;
		goto newstate;
	} /* switch */
    } /* if */

    /* reduction, given a production n */
#ifdef YYDEBUG
    printf("reduce with rule %d\n", n);
#endif
#ifdef YYASTFLAG
    if ( redCount<YYREDMAX )
	redSequence[redCount++] = n;
#endif
    yyps -= yyr2[n];
    yypvt = yypv;
    yypv -= yyr2[n];
    yyval = yypv[1];
    m = n;
    /* find next state from goto table */
    n = yyr1[n];
    j = yypgo[n] + *yyps + 1;
    if (j>=YYLAST || yychk[ tmpstate = yyact[j] ] != -n)
	tmpstate = yyact[yypgo[n]];
    switch (m)
    { /* actions associated with grammar rules */
	$A 
    }
    goto enstack;
}

/* This function will return the count of errors for a parser that is
generated by yyParse().
*/     
int ABXYacc::get_yyErrorCount(void)
{
	  return yyErrorCount;
}
				 
/* This function serves the purpose to allow a parser to choose 
the lexer function which provides tokens. The lexer function is 
specified as a ppinter to an integer function. The function will 
be called by the parser function.
*/   
void ABXYacc::yySetLexer(ABXLex *LexObject)
{
	lexObject = LexObject;
	yyToken = -1;
}

/* This function will modify the token stack size based on the input size
provided by user. It is a very important feature so that user can modify 
YACC buffer size to meat th eproject requirement.
*/
void ABXYacc::yySetTokStack(int stack_size)
{
	delete[] yyValueStack;
	delete[] redSequence;
	yyValueStack = new YYSTYPE[stack_size];
	redSequence = new int[stack_size];
}